(function ($) {

  Drupal.contextly = Drupal.contextly || {};

  /**
   * Generates widget class based on settings.
   */
  Drupal.contextly.WidgetFactory = {

    /**
     * Returns widget instance depending on the passed data or null if data is
     * empty or the widget type is unknown.
     *
     * @param widgetData
     *   Widget data returned by Contextly service.
     * @param moduleSettings
     *   Global settings passed from the module.
     *
     * @returns {Drupal.contextly.BaseWidget|null}
     */
    getWidget: function(widgetData, moduleSettings) {
      if (widgetData.type) {
        switch (widgetData.type) {
          case 'sidebar':
            return new Drupal.contextly.SidebarWidget(widgetData, moduleSettings);

          case 'normal':
            if (widgetData.settings && widgetData.settings.display_type) {
              switch (widgetData.settings.display_type) {
                case 'tabs':
                  return new Drupal.contextly.TabsWidget(widgetData, moduleSettings);

                case 'blocks':
                  return new Drupal.contextly.BlocksWidget(widgetData, moduleSettings);

                case 'default':
                  return new Drupal.contextly.TextWidget(widgetData, moduleSettings);
              }
            }
            break;
        }
      }

      return null;
    }

  };

  /**
   * Base class for all kinds of widgets.
   *
   * @class
   */
  Drupal.contextly.BaseWidget = Drupal.contextly.createClass( /** @lends Drupal.contextly.BaseWidget.prototype */ {

    /**
     * Widget constructor.
     *
     * @param widgetData
     *   Widget data and settings received from Contextly.
     * @param moduleSettings
     *   Global settings passed from the module.
     */
    construct: function (widgetData, moduleSettings) {
      this.widget = widgetData;
      this.settings = widgetData.settings;
      this.isIE7 = ($.browser.msie && parseFloat($.browser.version) < 8);
      this.version = moduleSettings.version;
      this.mode = moduleSettings.mode;

      // Basic state-dependent classes.
      this.classes = {
        empty: ['contextly-empty'],
        rendered: ['contextly-loaded']
      };
    },

    /**
     * Builds and returns HTML of the widget.
     *
     * @function
     *
     * @returns {string}
     */
    buildWidget: Drupal.contextly.abstractMethod(),

    /**
     * Returns HTML class identifying container of this widget.
     *
     * This class is not really unique on the page, because there can be
     * multiple instances of the same widget.
     *
     * @function
     *
     * @returns {string}
     */
    getWidgetClass: function() {
      return 'contextly-widget--' + this.widget.id;
    },

    /**
     * Returns true if passed in section can be displayed.
     *
     * @param {string} section
     *
     * @returns {boolean}
     */
    sectionIsDisplayable: function (section) {
      if ($.inArray(section, this.settings.display_sections) == -1) {
        // Section is hidden by settings.
        return false;
      }

      return !!(this.widget.links && this.widget.links[section] && this.widget.links[section].length > 0);
    },

    /**
     * Loads base CSS for the widget.
     */
    loadCSSTemplate: function() {
      // Do nothing by default.
    },

    /**
     * Loads IE7-specific CSS fixes.
     */
    loadIE7CSSFix: function() {
      // Do nothing by default.
    },

    /**
     * Attaches event handlers to the container after rendering HTML to it.
     *
     * @param container
     */
    attachEventHandlers: function(container) {
      this.attachDynamicURLHandlers(container);
    },

    /**
     * Attaches handlers to dynamically replace the link URL.
     *
     * URL of the link is replaced with the one stored in data-* attribute.
     *
     * Actual URL is a redirect URL on the Contextly server. On mouse over user
     * sees the final destination and on click we pass him through the redirect
     * to get some statistics.
     *
     * @param container
     */
    attachDynamicURLHandlers: function(container) {
      container
        .find('a.contextly_dynamic_url')
        .mousedown(function () {
          var $this = $(this);
          $this.attr('href', $this.attr('data-contextly-url'));
        });
    },

    /**
     * Returns true if widget has data to display.
     *
     * @returns {boolean}
     */
    hasData: function () {
      return !!(this.widget && this.widget.links && !$.isEmptyObject(this.widget.links));
    },

    /**
     * Adds classes to the container depending on the passed in state.
     *
     * @param {string} state
     *   One of:
     *   - empty
     *   - rendered
     * @param container
     */
    addStateClasses: function(state, container) {
      if (this.classes && this.classes[state]) {
        $.each(this.classes[state], function() {
          container.addClass(this);
        });
      }
    },

    /**
     * Removes classes to the container depending on the passed in state.
     *
     * @param {string} state
     *   One of:
     *   - empty
     *   - rendered
     * @param container
     */
    removeStateClasses: function(state, container) {
      if (this.classes && this.classes[state]) {
        $.each(this.classes[state], function() {
          container.removeClass(this);
        });
      }
    },

    /**
     * Displays the widget inside passed container.
     *
     * @param container
     *   jQuery object of the container tag.
     */
    display: function(container) {
      if (!container.size()) {
        return;
      }

      if (!this.hasData()) {
        // The widget is empty. Just add classes and exit.
        this.addStateClasses('empty', container);
        return;
      }

      // Load CSS template, IE 7 fixes and apply settings CSS.
      this.loadCSSTemplate();
      if (this.isIE7) {
        this.loadIE7CSSFix();
      }
      Drupal.contextly.CSSHelper.applySettingsOnce(this.getWidgetClass(), this.settings);

      // Render HTML into the container, attach event handlers and Drupal
      // behaviors.
      container.html(this.buildWidget());
      this.addStateClasses('rendered', container);
      this.attachEventHandlers(container);
      Drupal.attachBehaviors(container, Drupal.settings);
    }

  });

  /**
   * Text-only widget.
   *
   * @class
   * @extends Drupal.contextly.BaseWidget
   */
  Drupal.contextly.TextWidget = Drupal.contextly.createClass( /** @lends Drupal.contextly.TextWidget.prototype */ {

    /**
     * Parent class.
     */
    extend: Drupal.contextly.BaseWidget,

    /**
     * @inheritDoc
     */
    buildWidget: function () {
      var div = "";
      var value;

      div += "<div class='contextly_see_also'>";
      // Check for top links html
      if (value = this.settings.html_above) {
        // HTML allowed in this option.
        div += "<div class='contextly_html_above'>" + value + "</div>";
      }
      // Check for title
      if (value = this.settings.title) {
        // HTML allowed in this option.
        div += "<div class='contextly_title'>" + value + "</div>";
      }

      var sections = this.widget.settings.display_sections;
      for (var section in sections) {
        var section_name = sections[section];
        if (this.sectionIsDisplayable(section_name)) {
          div += "<div class='contextly_previous'>";
          div += "<span class='contextly_subhead'>" + this.widget.settings[section_name + '_subhead'] + "</span>";
          div += "<ul class='link'>" + this.buildSectionLinks(section_name) + "</ul>";
          div += "</div>";
        }
      }

      return div;
    },

    /**
     * Returns link HTML.
     *
     * @param link
     *
     * @returns {string}
     */
    buildLink: function (link) {
      var result = '<a class="title module-contextly contextly_dynamic_url" href="' + Drupal.checkPlain(link.native_url) + '" title="' + Drupal.checkPlain(link.title) + '" data-contextly-url="' + Drupal.checkPlain(link.url) + '">';
      result += Drupal.checkPlain(link.title);
      result += '</a>';

      if (this.isIE7) {
        result += '<!--[if lte ie 7]><b></b><![endif]-->';
      }

      return '<li>' + result + '</li>';
    },

    /**
     * Return HTML of the passed section links.
     *
     * Section must be checked if it is displayable before calling this
     * function.
     *
     * @param {string} section
     * @returns {string}
     */
    buildSectionLinks: function (section) {
      var html = "";

      for (var index in this.widget.links[section]) {
        var link = this.widget.links[section][index];
        html += this.buildLink(link);
      }

      return html;
    }

  });

  /**
   * Full-featured HTML widget class.
   *
   * @class
   * @extends Drupal.contextly.BaseWidget
   */
  Drupal.contextly.TabsWidget = Drupal.contextly.createClass( /** @lends Drupal.contextly.TabsWidget.prototype */ {

    /**
     * Parent class.
     */
    extend: Drupal.contextly.BaseWidget,

    /**
     * Attaches event handlers to switch between tabs.
     *
     * Each tab switch triggers Contextly event.
     *
     * First available tab is activated by this function.
     *
     * @param container
     */
    attachTabHandlers: function(container) {
      var self = this;

      var tabHeaders = container
        .find('ul.linker_tabs > li[id]');
      var tabContents = container
        .children('div.linker_content');

      // Activate first available tab.
      tabHeaders
        .first()
        .addClass('active');
      tabContents
        .first()
        .css({
          display: 'block'
        });

      // Attach event handlers.
      tabHeaders
        .find('a')
        .click(function () {
          var header = $(this).closest('li');
          if (header.is('.active')) {
            // Don't do anything if clicked tab is already active.
            return;
          }

          var section = header.attr('data-contextly-section');
          if (!section) {
            return;
          }

          var content_id = 'linker_content_' + section;
          var content = container.find('#' + content_id);
          if (!content.size()) {
            return;
          }

          // Reset tabs state.
          tabHeaders
            .filter('.active')
            .removeClass('active');
          tabContents.hide();

          // Activate clicked tab.
          header.addClass('active');
          content.css({
            display: 'block'
          });

          // Fire the event.
          Drupal.contextly.firePageEvent(self.widget.id, "switch_tab", section);
        });
    },

    /**
     * Attaches handlers for the "Show more/less" links.
     *
     * Each click on these links triggers Contextly event.
     *
     * @param container
     */
    attachShowMoreHandlers: function(container) {
      var self = this;

      container
        .find('p.show-more > a')
        .click(function() {
          var tab = $(this)
            .closest('div.linker_content')
            .attr('data-contextly-section');
          if (!tab) {
            return;
          }

          $('.li' + tab).toggleClass("show");

          var pMore = $('#pmore' + tab);
          var aMore = pMore.children('a');
          pMore.toggleClass("show");
          if (pMore.hasClass('show')) {
            aMore.text(Drupal.t('Show Less'));
          }
          else {
            aMore.text(Drupal.t('Show More'));
          }

          // Fire the event.
          Drupal.contextly.firePageEvent(self.widget.id, "show_more", tab);
        });
    },

    /**
     * @inheritDoc
     */
    attachEventHandlers: function (container) {
      this.attachDynamicURLHandlers(container);
      this.attachTabHandlers(container);
      this.attachShowMoreHandlers(container);
    },

    /**
     * @inheritDoc
     */
    loadIE7CSSFix: function() {
      var css_url = "https://c713421.ssl.cf2.rackcdn.com/_plugin/" + this.version + "/css/template-ie-fix.css";
      Drupal.contextly.CSSHelper.loadOnce(css_url);
    },

    /**
     * @inheritDoc
     */
    loadCSSTemplate: function() {
      var url;
      switch (this.mode) {
        case 'dev':
          url = "http://dev.contextly.com/resources/css/plugin/widget/" + this.settings.display_type + "/template-" + this.settings.tabs_style + ".css";
          break;

        default:
          url = "https://c713421.ssl.cf2.rackcdn.com/_plugin/" + this.version + "/css-api/widget/" + this.settings.display_type + "/template-" + this.settings.tabs_style + ".css";
      }
      Drupal.contextly.CSSHelper.loadOnce(url);

      // If this is IE7 we need to load IE7 css fix
      if (this.isIE7) {
        this.loadIE7CSSFix();
      }
    },

    /**
     * @inheritDoc
     */
    buildWidget: function () {
      var div = "";
      var sections = this.settings.display_sections;

      var headers = [],
        tabs = [];
      for (var section in sections) {
        var section_name = sections[section];

        if (this.sectionIsDisplayable(section_name)) {
          var section_key = section_name + '_subhead';
          var section_header = this.settings[ section_key ];

          var header = '<li id="linker_tab_' + Drupal.checkPlain(section_name) + '" data-contextly-section="' + Drupal.checkPlain(section_name) + '">';
          header += '<a href="javascript:;"><span>' + Drupal.checkPlain(section_header) + '</span></a>';
          header += '</li>';
          headers.push(header);

          var tab = '<div id="linker_content_' + Drupal.checkPlain(section_name) + '" class="linker_content" data-contextly-section="' + Drupal.checkPlain(section_name) + '">';
          tab += '<ul class="link' + ( this.sectionHasImages(section_name) ? ' linker_images' : '' ) + '">';
          tab += this.buildSectionLinks(section_name);
          tab += '</ul>';
          tab += '</div>';
          tabs.push(tab);
        }
      }

      if (value = this.settings.html_above) {
        // HTML allowed in this setting.
        div += '<span class="contextly_above_related">' + value + '</span>';
      }

      div += '<ul class="linker_tabs">';
      div += headers.join('');

      if (this.contextlyLogoIsDisplayable()) {
        div += '<li><span class="contextly_related"><a href="http://contextly.com">' + Drupal.t('Related Links by Contextly') + '</a></span></li>';
      }

      div += "</ul>";

      div += tabs.join('');

      return div;
    },

    /**
     * Return HTML of the passed section links.
     *
     * Section must be checked if it is displayable before calling this
     * function.
     *
     * @param {string} section
     * @returns {string}
     */
    buildSectionLinks: function (section) {
      var html = "";

      for (var index in this.widget.links[section]) {
        var link = this.widget.links[section][index];

        html += '<li';
        if (parseInt(index) + 1 > this.settings.links_limit) {
          html += ' class="li' + Drupal.checkPlain(section) + '"';
        }
        html += '>';
        html += this.buildLink(link);
        html += '</li>';
      }

      if (this.widget.links[ section ].length > this.settings.links_limit) {
        html += '<p class="show-more" id="pmore' + Drupal.checkPlain(section) + '"><a href="javascript:;" name="amore' + Drupal.checkPlain(section) + '">' + Drupal.t('Show More') + '</a></p>';
      }

      return html;
    },

    /**
     * Returns link HTML.
     *
     * @param link
     *
     * @returns {string}
     */
    buildLink: function (link) {
      // TODO Move to CSS file!
      var itemStyle = "padding-bottom: 5px;";

      if (link.thumbnail_url) {
        itemStyle += "height: " + this.getImagesHeight() + "px;";
      }

      var html = '<ul class="horizontal-line" style="' + Drupal.checkPlain(itemStyle) + '">';

      var linkStart = '<a class="title module-contextly contextly_dynamic_url" href="' + Drupal.checkPlain(link.native_url) + '" title="' + Drupal.checkPlain(link.title) + '" data-contextly-url="' + Drupal.checkPlain(link.url) + '">';
      var ieFix = this.isIE7 ? '<!--[if lte ie 7]><b></b><![endif]-->' : '';

      if (link.thumbnail_url) {
        var imageItemStyle = 'width: ' + (this.getImagesWidth() + 8) + 'px;';
        html += '<li style="' + Drupal.checkPlain(imageItemStyle) + '">' + linkStart + '<img src="' + Drupal.checkPlain(link.thumbnail_url) + '" /></a>' + ieFix + '</li>';
      }

      html += "<li>" + linkStart + Drupal.checkPlain(link.title) + "</a>";
      if (this.settings.display_link_dates && link.publish_date) {
        html += ' <span class="link-pub-date">' + this.dateTextRepresentation(link.publish_date) + '</span>';
      }
      html += ieFix + '</li>';

      html += '</ul>';

      return html;
    },

    /**
     * Returns text representation of the time range between current time and
     * passed date.
     *
     * @param date
     * @returns {string}
     */
    dateTextRepresentation: function (date) {
      if (date && date.length > 4) {
        var t = date.split(/[- :]/);
        var js_date = new Date(t[0], t[1] - 1, t[2], t[3], t[4], t[5]);

        var timestamp = js_date.getTime() / 1000;
        var difference = new Date().getTime() / 1000 - timestamp;

        var periods = new Array("sec", "min", "hour", "day", "week", "month", "year", "decade");
        var lengths = new Array("60", "60", "24", "7", "4.35", "12", "10");
        var ending;

        if (difference > 0) {
          // this was in the past
          ending = "ago";
        }
        else { // this was in the future
          return 'right now';
        }

        for (var j = 0; difference >= lengths[j]; j++) {
          difference /= lengths[j];
        }
        difference = Math.round(difference);

        if (difference != 1) {
          periods[j] += "s";
        }
        return difference + "&nbsp;" + periods[j] + "&nbsp;" + ending;
      }

      return '';
    },

    /**
     * Returns true if passed section has images.
     *
     * @param {string} section
     * @returns {boolean}
     */
    sectionHasImages: function (section) {
      var imgCount = 0;

      for (var index in this.widget.links[section]) {
        if (this.widget.links[section][index].thumbnail_url) {
          imgCount++;
        }
      }

      // TODO What about using threshold from settings?
      if (this.widget.links[ section ].length == imgCount) {
        return true;
      }
      else {
        return false;
      }
    },

    /**
     * Returns height of the images.
     *
     * @returns {number}
     */
    getImagesHeight: function () {
      switch (this.settings.images_type) {
        case 'square32x32':
        case 'letter82x32':
          return 32;
        case 'square45x45':
        case 'letter82x45':
          return 45;
        case 'square90x90':
        case 'letter110x90':
          return 90;
        case 'square70x70':
          return 70;
        case 'square110x110':
          return 110;
        case 'square150x150':
          return 150;
        default:
          return 0;
      }
    },

    /**
     * Returns width of the images.
     *
     * @returns {number}
     */
    getImagesWidth: function () {
      switch (this.settings.images_type) {
        case 'letter82x32':
        case 'letter82x45':
          return 82;
        case 'square110x110':
        case 'letter110x90':
          return 110;
        case 'square32x32':
          return 32;
        case 'square45x45':
          return 45;
        case 'square70x70':
          return 70;
        case 'square90x90':
          return 90;
        case 'square150x150':
          return 150;
        default:
          return 0;
      }
    },

    /**
     * Returns true if Contextly logo must be displayed on the
     *
     * @returns {boolean}
     */
    contextlyLogoIsDisplayable: function () {
      return !this.isMobileRequest();
    },

    /**
     * Returns true if we are on mobile device.
     *
     * @returns {boolean}
     */
    isMobileRequest: function () {
      return false; // TODO: implement me
    }

  });

  /**
   * HTML Blocks widget class.
   *
   * @class
   * @extends Drupal.contextly.TextWidget
   */
  Drupal.contextly.BlocksWidget = Drupal.contextly.createClass({

    /**
     * Parent class.
     */
    extend: Drupal.contextly.TextWidget,

    /**
     * @inheritDoc
     */
    buildWidget: function () {
      var div = "";
      var value;

      div += "<div class='contextly_see_also blocks-widget'>";
      // Check for top links html
      if (value = this.settings.html_above) {
        div += "<div class='contextly_html_above'>" + value + "</div>";
      }
      // Check for title
      if (value = this.settings.title) {
        div += "<div class='contextly_title'>" + value + "</div>";
      }

      var sections = this.settings.display_sections;

      div += "<div class='contextly_around_site'>";
      for (var section in sections) {
        var section_name = sections[section];

        if (this.sectionIsDisplayable(section_name)) {
          var section_key = section_name + '_subhead';
          var section_header = this.settings[ section_key ];

          div += "<div class='contextly_previous'>";
          div += "<span class='contextly_subhead'>" + section_header + "</span>";
          div += "<ul class='link'>" + this.buildSectionLinks(section_name) + "</ul>";
          div += "</div>";
        }
      }
      div += "</div>";

      return div;
    },

    buildLink: function (link) {
      var result = '<p><span>';
      result += Drupal.checkPlain(link.title);
      result += '</span></p>';

      if (link.thumbnail_url) {
        result += '<img src="' + Drupal.checkPlain(link.thumbnail_url) + '" />';
      }

      // Wrap with the link.
      result = '<a href="' + Drupal.checkPlain(link.native_url) + '" title="' + Drupal.checkPlain(link.title) + '" data-contextly-url="' + Drupal.checkPlain(link.url) + '" class="contextly_dynamic_url">' + result + '</a>';

      if (this.isIE7) {
        result += '<!--[if lte ie 7]><b></b><![endif]-->';
      }

      return '<li>' + result + '</li>';
    },

    /**
     * @inheritDoc
     */
    loadCSSTemplate: function () {
      var url = '';
      switch (this.mode) {
        case 'dev':
          url = "http://dev.contextly.com/resources/css/plugin/widget/blocks/template-default.css";
          break;

        default:
          url = "https://c713421.ssl.cf2.rackcdn.com/_plugin/" + this.version + "/css-api/widget/blocks/template-default.css";
          break;
      }
      Drupal.contextly.CSSHelper.loadOnce(url);
    }

  });

  /**
   * Sidebar widget class.
   *
   * @class
   * @extends Drupal.contextly.TabsWidget
   */
  Drupal.contextly.SidebarWidget = Drupal.contextly.createClass( /** @lends Drupal.contextly.SidebarWidget.prototype */ {

    /**
     * Parent class.
     */
    extend: Drupal.contextly.TabsWidget,

    /**
     * @inheritDoc
     */
    buildWidget: function () {
      var value,
        html = '';
      html += '<div class="linker_content">';

      // Add sidebar title and description if they are not empty.
      if (value = this.widget.name) {
        html += '<div class="title">' + Drupal.checkPlain(value) + '</div>';
      }
      if (value = this.widget.description) {
        // TODO: Description is a multi-line area, we should probably convert
        // newlines to <br>.
        html += '<div class="description">' + Drupal.checkPlain(value) + '</div>';
      }

      // Add the list of links.
      html += '<ul class="link' + ( this.sectionHasImages('previous') ? ' linker_images' : '' ) + '">';
      html += this.buildSectionLinks('previous');
      html += '</ul>';

      // Close the wrapper.
      html += '</div>';

      return html;
    },

    /**
     * @inheritDoc
     */
    buildSectionLinks: function (section) {
      var html = "";
      if (this.widget && this.widget.links && this.widget.links[section]) {
        for (var index in this.widget.links[section]) {
          var link = this.widget.links[section][index];

          if (link.id && link.title) {
            html += "<li>" + this.buildLink(link) + "</li>";
          }
        }
      }

      return html;
    },

    /**
     * @inheritDoc
     */
    loadCSSTemplate: function () {
      var url = '';
      switch (this.mode) {
        case 'dev':
          url = "http://dev.contextly.com/resources/css/plugin/sidebar/template-" + this.settings.theme + ".css";
          break;

        default:
          url = "https://c713421.ssl.cf2.rackcdn.com/_plugin/" + this.version + "/css-api/sidebar/template-" + this.settings.theme + ".css";
          break;
      }
      Drupal.contextly.CSSHelper.loadOnce(url);
    },

    /**
     * @inheritDoc
     */
    attachEventHandlers: function (container) {
      this.attachDynamicURLHandlers(container);
    }

  });

  /**
   * Helper object to build CSS from the widget settings.
   */
  Drupal.contextly.CSSHelper = {

    /**
     * The list of already loaded external CSS URLs.
     */
    loadedURLs: {},

    /**
     * Initiates loading of external CSS file with passed in URL.
     *
     * @param {string} url
     */
    loadOnce: function (url) {
      // Make sure we're not going to load the same CSS file again.
      if (this.loadedURLs[url]) {
        return;
      }

      // Mark URL as loaded.
      this.loadedURLs[url] = true;

      // Load it.
      $('<link>')
        .attr({
          rel: "stylesheet",
          media: "screen",
          type: "text/css",
          href: url
        })
        .appendTo('head');
    },

    /**
     * The list of widget classes with already applied settings CSS.
     */
    processedClasses: {},

    /**
     * Builds CSS rules from the widget settings and adds them to the head.
     *
     * For each widget class this is done only once.
     *
     * @param widgetClass
     *   Class that identifies the widget.
     * @param settings
     *   Widget settings returned by Contextly.
     */
    applySettingsOnce: function(widgetClass, settings) {
      // Make sure we're not going to build the same settings again.
      if (this.processedClasses[widgetClass]) {
        return;
      }

      // Mark class as already processed.
      this.processedClasses[widgetClass] = true;

      // Build the code and inject it to the page head.
      var css = this.buildSettingsCSS('.' + widgetClass, settings);
      if (css) {
        $('<style type="text/css" media="screen">' + css + "</style>")
          .appendTo('head');
      }
    },

    /**
     * Builds CSS code from passed widget settings to insert into the head.
     *
     * @param {string} prefix
     * @param settings
     *
     * @returns {string}
     */
    buildSettingsCSS: function (prefix, settings) {
      var code = "";

      if (settings.csscode) {
        // TODO Escape it!
        // See https://www.owasp.org/index.php/XSS_(Cross_Site_Scripting)_Prevention_Cheat_Sheet
        code += settings.csscode;
      }

      // TODO Collapse this spaghetti into definition object and loop.
      if (settings.fontfamily) {
        code += this.buildCSSRule(prefix, ".link a", "font-family", settings.fontfamily);
      }
      if (settings.fontsize) {
        code += this.buildCSSRule(prefix, ".link a", "font-size", settings.fontsize);
      }

      if (settings.font_family) {
        code += this.buildCSSRule(prefix, ".link a", "font-family", settings.font_family);
      }
      if (settings.font_size) {
        code += this.buildCSSRule(prefix, ".link a", "font-size", settings.font_size);
      }

      if (settings.color_background) {
        code += this.buildCSSRule(prefix, ".linker_content", "background-color", settings.color_background);
        code += this.buildCSSRule(prefix, ".linker_images img", "border-color", settings.color_background);
      }

      if (settings.color_links) {
        code += this.buildCSSRule(prefix, ".linker_content a", "color", settings.color_links);
        code += this.buildCSSRule(prefix, ".linker_content a:hover", "color", settings.color_links);
        code += this.buildCSSRule(prefix, ".linker_content span", "color", settings.color_links);
      }

      if (settings.color_border) {
        code += this.buildCSSRule(prefix, ".linker_content", "border-color", settings.color_border);
        code += this.buildCSSRule(prefix, ".linker_tabs li.active span", "border-color", settings.color_border);
        code += this.buildCSSRule(prefix, ".linker_tabs span", "border-color", settings.color_border);
      }

      if (settings.color_active_tab) {
        code += this.buildCSSRule(prefix, ".linker_tabs li.active span", "background-color", settings.color_active_tab);
      }

      if (settings.color_inactive_tab) {
        code += this.buildCSSRule(prefix, ".linker_tabs span", "background-color", settings.color_inactive_tab);
      }

      return code;
    },

    /**
     * Returns single CSS rule.
     *
     * @param {string} prefix
     *   Selector prefix.
     * @param {string} selector
     * @param {string} property
     *   Property name.
     * @param {string} value
     *   Property value.
     *
     * @returns {string}
     */
    buildCSSRule: function (prefix, selector, property, value) {
      if (!value) {
        return "";
      }

      // TODO Escape value!
      return prefix + " " + selector + " {" + property + ": " + value + "}\n";
    }

  };

  /**
   * Fires Contextly page event.
   *
   * @param {string} name
   *   Event name.
   * @param {string} [key]
   *   Event parameter value.
   */
  Drupal.contextly.firePageEvent = function(widgetId, name, key) {
    $(window).triggerHandler('contextlyPageEvent', {
      widgetId: widgetId,
      name: name,
      key: typeof key !== 'undefined' ? key : null,
      date: new Date()
    });
  };

  /**
   * Loads settings of all page snippets on the node page and displays them.
   *
   * Also, registers the page events tracking handler which submits the list
   * of events to the Contextly.
   */
  Drupal.behaviors.contextlyWidgets = {
    attach: function (context, settings) {
      if (!settings.contextlyWidgets) {
        return;
      }

      // Make sure we initialize widgets only once on the page.
      var isFirstTime = $('body', context)
        .once('contextly-widgets')
        .size();
      if (!isFirstTime) {
        return;
      }

      var s = $.extend({
        nid: 0,
        version: '',
        mode: '',
        sitePath: '',
        providerURL: null,
        settingsURL: null,
        pageEventsURL: null
      }, settings.contextlyWidgets);

      if (!s.nid || !s.version || !s.sitePath || !s.providerURL || !s.settingsURL) {
        return;
      }

      // Get widget containers and make sure they're not empty.
      var containers = $('.contextly-placeholder', context);
      if (!containers.size()) {
        return;
      }

      /**
       * Contextly events listener.
       *
       * Submits event data to the Contextly service.
       *
       * @param e
       * @param data
       */
      var onPageEvent = function(e, data) {
        var event = {
          site_path: s.sitePath,
          post_id: s.nid,
          setting_id: data.widgetId,
          event_name: data.name,
          event_key: data.key,
          event_date: data.date
        };
        restCall(s.pageEventsURL, event);
      };

      var attachPageEventHandlers = function() {
        $(window).bind('contextlyPageEvent', onPageEvent);
      };

      var onSettingsLoadingSuccess = function(data) {
        // Remove loading class.
        containers
          .removeClass('contextly-loading contextly-placeholder')
          .addClass('contextly-widget');

        // Attach page event handlers.
        if (s.pageEventsURL) {
          attachPageEventHandlers();
        }

        // Render snippets.
        $.each(data.entry.snippets, function() {
          var widgetData = this;
          var container = containers
            .filter('.contextly-snippet-placeholder:first')
            .removeClass('contextly-snippet-placeholder')
            .addClass('contextly-snippet contextly-widget--' + widgetData.id);
          var widget = Drupal.contextly.WidgetFactory.getWidget(widgetData, s);
          if (widget) {
            widget.display(container);

            // Remove processed container from the list of pending ones.
            containers = containers.not(container);
          }

          // For now we support only one snippet.
          return false;
        });

        // Render sidebars.
        $.each(data.entry.sidebars, function() {
          var widgetData = this;
          var selector = '.contextly-widget--' + widgetData.id;
          var sidebarContainers = containers
            .filter(selector)
            .removeClass('contextly-sidebar-placeholder')
            .addClass('contextly-sidebar contextly-sidebar-' + widgetData.layout);
          sidebarContainers.each(function() {
            var container = $(this);
            var widget = Drupal.contextly.WidgetFactory.getWidget(widgetData, s);
            if (widget) {
              widget.display(container);

              // Remove processed containers from the list of pending ones.
              containers = containers.not(container);
            }
          });
        });

        // Remove placeholders left unprocessed.
        containers.remove();
      };

      var onSettingsLoadingFailure = function() {
        containers
          .removeClass('contextly-loading')
          .addClass('contextly-error');
      };

      var loadSettings = function() {
        containers.addClass('contextly-loading');
        restCall(s.settingsURL, {
          site_path: s.sitePath,
          version: s.version,
          page_id: s.nid,
          admin: 0
        }, function (response) {
          if (response.data) {
            var data = easyXDM.getJSONObject().parse(response.data);
            if (data.success && data.entry) {
              onSettingsLoadingSuccess(data);
              return;
            }
          }

          // There was no data if we got here.
          onSettingsLoadingFailure();
        });
      };

      var rpc = null;
      var initRpc = function () {
        rpc = new easyXDM.Rpc({
          remote: s.providerURL
        },
        {
          remote: {
            request: {}
          }
        });
      };

      var restCall = function(url, params, callback) {
        if (!rpc) {
          // Create RPC proxy only once per page.
          initRpc();
        }

        // Make sure we pass at least blank function as a callback.
        if (typeof callback !== 'function') {
          callback = function() {};
        }

        // Make the request.
        rpc.request({
          url: url,
          method: "POST",
          data: params
        }, callback);
      };

      loadSettings();
    }
  };

})(jQuery);
